# Examples from Stubborn APIs Webinar

This project provides examples that show how to work with various (REST) APIs showing common patterns of API calls in CloverDX.

## Requirements

* CloverDX Server 5.6 or newer.
* To use HubSpot examples, you will need a valid HubSpot account and HubSpot API key.

## Installation

1. Deploy the project as CloverDX Server project. This can be done by first importing the project into CloverDX
Designer, then right-clicking the project and selecting _Convert to Server Project_ option from the context menu.
2. Configure Server APIs settings in the project. These settings can be found in parameter file `conf/cloverdx-server.prm`:
	1. Set `SERVER_BASE_URL` to base URL of your CloverDX Server. This will usually be something like
	`http://localhost:8083` or similar.
	2. Set `SERVER_USER` and `SERVER_PASS` to user name and password for your Server. Note that the password is a
	secure parameter and you have to have secure parameters enabled and configured on your Server before using it. To
	enable and configure secure parameters on your Server, consult our
	[documentation](https://doc.cloverdx.com/latest/server/secure-parameters.html).
3. Configure HubSpot API settings.
	1. Review settings in `conf/hubspot.prm`. Typically these settings do not need changing but you need to confirm
	the HubSpot API URL.
	2. In `conf/hubspot-auth.prm` you need to change the account number (in `HUBSPOT_ACCOUNT_ID` parameter) to
	match your HubSpot account and `HUBSPOT_API_KEY` parameter to your API key.
4. Deploy Data Service `data-service/UploadFile.rjob`. This can be done by right-clicking the file in Navigator and
selecting context menu option _Data Services_ -> _Publish Data Services_.
5. To run the "01", "02" and "03" examples (which all query the Server for graph status of several jobs), you need to
update the file `data-in/run_ids.txt`. This file contains run ids of jobs to query - one id per line. Values to query
can be found in the Server console in _Execution history_ module (any jobs from the module will work).
6. To run the "04" example (which queries HubSpot API for company data), you need to update `data-in/companies.txt`
file. This file contains ids of multiple HubSpot company entities - one entity per line. To get these ids, find any
number of companies in your HubSpot instance and get their ids from their URL. For example, if the URL of a company
entity is `https://app.HubSpot.com/contacts/1234567/company/1234567890/`, the id to put into the file is the
last part of the URL: `1234567890`. You can specify any number of such ids in the file.

## Running the Examples

All examples work as stand-alone graphs that call various subgraphs as connectors. To run each example, simply open it
in Designer and start the graph (either from toolbar or by pressing Ctrl+R).

These examples are meant to be examined in detail in the Designer. After running each, review what data is sent between
different components or try debugging the CTL code to get even more detail (especially in more complex HubSpot examples).

As an exercise, we recommend extending the API examples to query data instead of using entity ids in various files. For
the three examples that check graph status, you can use
[Simple HTTP API](https://doc.cloverdx.com/latest/server/simple-http-api.html) to check the currently or previously
running graphs. Similarly, HubSpot offers [APIs](https://legacydocs.hubspot.com/docs/overview) that allow you to search
and list existing entities.