
// Trim string and return null if the result is empty string.
function string trimToNull(string s) {
	string r = trim(s);
	if (r == "") {
		return null;
	}
	return r;
}

// Same as trimToNull, but convert result to upper case.
function string trimToNullUp(string s) {
	string r = trim(s);
	if (r == "") {
		return null;
	}

	return upperCase(r);
}

// Same as trimToNull, but convert result to lower case.
function string trimToNullLow(string s) {
	string r = trim(s);
	if (r == "") {
		return null;
	}

	return lowerCase(r);
}

function string[] trimToNull(string[] s) {
	if (s == null) {
		return null;
	}
	
	string[] result;
	foreach (string t: s) {
		push(result, trimToNull(t));
	}
	
	return result;
}

function string[] trimToNullLow(string[] s) {
	if (s == null) {
		return null;
	}
	
	string[] result;
	foreach (string t: s) {
		push(result, trimToNullLow(t));
	}
	
	return result;
}

function string[] trimToNullUp(string[] s) {
	if (s == null) {
		return null;
	}
	
	string[] result;
	foreach (string t: s) {
		push(result, trimToNullUp(t));
	}
	
	return result;
}

// Remove characters specified in chars from the input string.
function string trimChars(string input, string chars) {
	integer start = 0;
	integer end = length(input) - 1;

	while (start < length(input)) {
		if (indexOf(chars, charAt(input, start)) < 0) {
			break;
		}
		start++;
	}
	
	if (start < length(input) - 1) {
		while (end > start) {
			if (indexOf(chars, charAt(input, end)) < 0) {
				break;
			}
			end--;
		}
	}
	
	return substring(input, start, end - start + 1);
}

// Concatenate all non-empty and non-null strings from the input list. List entries will be separated
// by provided delimiter. The input elements are not trimmed automatically. 
function string joinNonEmpty(string delimiter, string[] input) {
	return joinNonEmpty(delimiter, input, false);
}

// Concatenate all non-empty and non-null strings from the input list. List entries will be separated
// by provided delimiter. If autoTrim is set to true, all field values are trimmed (via trimToNull)
// before they are compared with null.
function string joinNonEmpty(string delimiter, string[] input, boolean autoTrim) {
	if (input == null) {
		return null;
	}
	
	string[] nonEmpty;
	foreach (string s: input) {
		if (autoTrim) {
			s = trimToNull(s);
		}
		if (s != null && !isBlank(s)) {
			push(nonEmpty, s);
		}
	}
	
	if (length(nonEmpty) == 0) {
		return null;
	}

	return join(delimiter, nonEmpty);
}

// Get rid of all white-space characters and replace them with regular spaces. Also replace runs of multiple white-spaces with a single space.
function string collapseSpaces(string input) {
	return replace(input, "\\s+", " ");
}

function string safeCharAt(string s, integer pos) {
	if (s == null) {
		return null;
	}
	
	if (pos < 0 || pos >= length(s)) {
		return null;
	}
	
	return charAt(s, pos);
}

// Concatenate list elements without modifying any list.
function string[] concatList(string[] a, string[] b) {
	string[] result;
	
	if (a != null) {
		result = copy(result, a);
	}
	
	if (b != null) {
		result = copy(result, b);
	}
	
	return result;
}

function map[string, integer] makeMapSet(string[] keys) {
	map[string, integer] result;
	
	if (keys == null || length(keys) == 0) {
		return result;
	}
	
	foreach (string key: keys) {
		result[key] = nvl(result[key], 0) + 1;
	}
	
	return result;
}

function string prettyPrintMap(map[string, integer] m, string keyValueSeparator, string entrySeparator) {
	string[] keyValuePairs;
	string[] keys = getKeys(m);
	
	keys = sort(keys);
	
	foreach (string k: keys) {
		push(keyValuePairs, k + keyValueSeparator + m[k]);
	}
	
	return join(entrySeparator, keyValuePairs);
}

function string prettyPrintList(string[] l, string elemSeparator) {
	return join(elemSeparator, l);
}
